@@ -50,7 +50,14 @@ process_cosinor <- function(formula){
   
   # Final expanded formula
   final_terms <- c(mainpart, expanded_terms)
-  expanded_formula <- stats::as.formula(paste("~", paste(final_terms, collapse = " + ")))
+  
+  expanded_formula <- if (length(final_terms) == 0) {
+    ~1
+  } else {
+    stats::as.formula(paste("~", paste(final_terms, collapse = " + ")))
+  }
+  
+  # expanded_formula <- stats::as.formula(paste("~", paste(final_terms, collapse = " + ")))
   
   expanded_formula
 }
@@ -130,8 +137,7 @@ cosinor = function(x = 1:24, period = 24, eval = TRUE){
 #' Note that for tensorproduct smooths, the corresponding list entry is itself a list, containing the d marginal penalty matrices if d is the dimension of the tensor product.
 #' @export
 #' 
-#' @importFrom mgcv gam
-#' @importFrom mgcv s
+#' @importFrom mgcv gam s
 #' @importFrom stats update
 #'
 #' @examples
@@ -147,7 +153,7 @@ cosinor = function(x = 1:24, period = 24, eval = TRUE){
 #' modmat = make_matrices(~ s(g, bs = "re") + s(x, bs = "ps"), data)
 #' # tensorproduct of x and y
 #' modmat = make_matrices(~ s(x) + s(y) + ti(x,y), data)
-make_matrices = function(formula, 
+make_matrices_old = function(formula, 
                          data, 
                          knots = NULL
                          ){
@@ -156,34 +162,19 @@ make_matrices = function(formula,
   formula = process_cosinor(formula)
   
   ## setting up the model using mgcv
-  gam_setup = mgcv::gam(formula = stats::update(formula, dummy ~ .),
-                        data = cbind(dummy = 1, data), 
-                        knots = knots,
-                        fit = FALSE)
-  
+  gam_setup = gam(formula = update(formula, dummy ~ .),
+                  data = cbind(dummy = 1, data), 
+                  knots = knots,
+                  fit = FALSE)
   
   ## assiging design matrix
   term_names = gam_setup$term.names
   Z = gam_setup$X
   colnames(Z) = term_names
 
-  
   ## dealing with the penalty matrices
   term_labels = sapply(gam_setup$smooth, function(x) x$label)
   
-  # first option: tensorproducts -> save marginal penalty matrices
-  # S2 = lapply(gam_setup$smooth, function(x){
-  #   if(is.null(x$margin)){ # univariate smooth -> one penalty matrix
-  #     return(x$S[[1]])
-  #   } else{ # multivariate smooth -> several penalty matrices
-  #     S_sublist = lapply(x$margin, function(y) y$S[[1]])
-  #     margin_names = sapply(x$margin, function(y) y$term)
-  #     names(S_sublist) = margin_names
-  #     return(S_sublist)
-  #   }
-  # })
-  # names(S2) = term_labels
-  
   # second option: tensorproduct -> save blown-up marginal penalty matrices (with constraints baked in)
   S = list()
   counter = 1
@@ -215,7 +206,6 @@ make_matrices = function(formula,
   
   pardim = c(pardim, pardim_smooth)
   
-  
   out = list(Z = Z, 
              S = S, 
              # S2 = S2,
@@ -229,20 +219,452 @@ make_matrices = function(formula,
   return(out)
 }
 
+make_matrices_flat <- function(formula, data, knots = NULL) {
+  
+  get_name <- function(fml, idx) {
+    if (length(fml) == 3 && !deparse(fml[[2]]) %in% c("", ".")) {
+      deparse(fml[[2]])
+    } else {
+      paste0("par", idx)
+    }
+  }
+  
+  process_single <- function(fml, name, knots_sub) {
+    fml <- process_cosinor(fml)
+    
+    # prepare gam model setup
+    gam_setup <- gam(update(fml, dummy ~ .), data = cbind(dummy = 1, data), 
+                   knots = knots_sub, fit = FALSE)
+    
+    # also prepare prediction gam
+    gam_setup0 <- gam(update(fml, dummy ~ .), data = cbind(dummy = 1, data), 
+                   knots = knots_sub, control = list(maxit = 1))
+    
+    Z <- gam_setup$X
+    colnames(Z) <- gam_setup$term.names
+    
+    S <- list()
+    coef <- list()
+    pardim <- list(fixed_eff = gam_setup$nsdf)
+    counter <- 1
+    
+    for (i in seq_along(gam_setup$smooth)) {
+      sm <- gam_setup$smooth[[i]]
+      if(is.null(name)){
+        label <- sm$label
+      } else {
+        label <- paste0(name, ".", sm$label)
+      }
+      
+      if (is.null(sm$margin)) {
+        # Single-penalty smooth
+        S[[label]] <- gam_setup$S[[counter]]
+        pardim[[sm$label]] <- nrow(S[[label]])
+        coef[[label]] <- rep(0, nrow(S[[label]]))
+        counter <- counter + 1
+      } else {
+        # Tensor product smooth
+        n_pen <- length(sm$margin)
+        margin_names <- sapply(sm$margin, `[[`, "term")
+        penalty_list <- gam_setup$S[counter:(counter + n_pen - 1)]
+        names(penalty_list) <- margin_names
+        S[[label]] <- penalty_list
+        pardim[[sm$label]] <- nrow(penalty_list[[1]])
+        coef[[label]] <- rep(0, nrow(penalty_list[[1]]))
+        counter <- counter + n_pen
+      }
+    }
+    
+    coef_fixed <- rep(0, gam_setup$nsdf)
+    names(coef_fixed) <- colnames(Z)[seq_len(gam_setup$nsdf)]
+    # coef <- c(setNames(list(coef_fixed), paste0(name, ".fixed_eff")), coef)
+    
+    list(
+      Z = Z, S = S, pardim = pardim,
+      gam = gam_setup, gam0 = gam_setup0,
+      knots = knots_sub, coef = coef
+    )
+  }
+  
+  if (!inherits(formula, "list")) {
+    name <- get_name(formula, 1)
+    if(name == "par1"){
+      name <- NULL # default name for single formula
+    }
+    res <- process_single(formula, name, knots)
+    out <- list(
+      Z = res$Z, S = res$S, 
+      pardim = res$pardim,
+      coef = res$coef,
+      data = data, 
+      gam = res$gam, gam0 = res$gam0,
+      knots = knots
+    )
+    class(out) <- "LaMa_matrices"
+    return(out)
+  }
+  
+  # get names
+  form_names <- names(formula) # also allow for named lists with right-side-only formulas
+  if(is.null(form_names)){
+    form_names <- sapply(seq_along(formula), function(i) get_name(formula[[i]], i))
+  }
+  
+  # check knots
+  if(!is.null(knots)){
+    if(!all(names(knots) %in% form_names)){
+      stop("Names of 'knots' must match the names of the formulas in 'formula'.")
+    }
+  }
+  
+  # Case: list of formulas
+  Z_list <- list()
+  S_flat <- list()
+  pardim_list <- list()
+  formula_list <- list()
+  knots_list <- list()
+  gam_list <- list()
+  gam0_list <- list()
+  coef_list <- list()
+  
+  # formula_names <- names(formula)
+  
+  for (i in seq_along(formula)) {
+    fml <- formula[[i]]
+    name <- form_names[i]
+    res <- process_single(fml, name, knots[[name]])
+    
+    Z_list[[name]] <- res$Z
+    S_flat <- c(S_flat, res$S)
+    pardim_list[[name]] <- res$pardim
+    knots_list[[name]] <- res$knots
+    gam_list[[name]] <- res$gam
+    gam0_list[[name]] <- res$gam0
+    coef_list <- c(coef_list, res$coef)
+  }
+  
+  out <- list(
+    Z = Z_list, S = S_flat, 
+    pardim = pardim_list,
+    coef = coef_list,
+    data = data,
+    gam = gam_list, gam0 = gam0_list,
+    knots = knots_list
+  )
+  class(out) <- "LaMa_matrices"
+  return(out)
+}
+
+#' Build the design and the penalty matrix for models involving penalised splines based on a formula and a data set
+#'
+#' @param formula formula as used in \code{mgcv}. Formulas can be right-side only, or contain a response variable, which is just extracted for naming.
+#'
+#' Can also be a list of formulas, which are then processed separately. In that case, both a named list of right-side only formulas or a list of formulas with response variables can be provided.
+#' @param data data frame containing all the variables on the right side of the formula(s)
+#' @param knots optional list containing user specified knot values for each covariate to be used for basis construction.
+#' For most bases the user simply supplies the \code{knots} to be used, which must match up with the \code{k} value supplied (note that the number of knots is not always just \code{k}).
+#' See \code{mgcv} documentation for more details.
+#'
+#' If \code{formula} is a list, this needs to be a named (based on the response variables) list over such lists.
+#' 
+#' @seealso \code{\link{predict.LaMa_matrices}} for prediction design matrix construction based on the model matrices object created by this function.
+#'
+#' @return a list of class \code{LaMa_matrices} containing:
+#' \item{\code{Z}}{design matrix (or list of such matrices if \code{formula} is a list))}
+#' \item{\code{S}}{list of penalty matrices (with names based on the response terms of the formulas as well as the smooth terms and covariates). For tensorproduct smooths, corresponding entries are themselves lists, containing the \eqn{d} marginal penalty matrices if \eqn{d} is the dimension of the tensor product)}
+#' \item{\code{pardim}}{list of parameter dimensions (fixed and penalised separately) for each formula, for ease of setting up initial parameters}
+#' \item{\code{coef}}{list of coefficient vectors filled with zeros of the correct length for each formula, for ease of setting up initial parameters}
+#' \item{\code{data}}{the data frame used for the model(s)}
+#' \item{\code{gam}}{unfitted \code{mgcv::gam} object used for construction of \code{Z} and \code{S} (or list of such objects if \code{formula} is a list)}
+#' \item{\code{gam0}}{fitted \code{mgcv::gam} which is used internally for to create prediction design matrices (or list of such objects if \code{formula} is a list)}
+#' \item{\code{knots}}{knot list used in the basis construction (or named list over such lists if \code{formula} is a list}
+#'
+#' @export
+#' 
+#' @importFrom mgcv gam s
+#' @importFrom stats update
+#'
+#' @examples
+#' data = data.frame(x = runif(100), 
+#'                   y = runif(100),
+#'                   g = factor(rep(1:10, each = 10)))
+#'
+#' # unvariate thin plate regression spline
+#' modmat = make_matrices(~ s(x), data)
+#' # univariate P-spline
+#' modmat = make_matrices(~ s(x, bs = "ps"), data)
+#' # adding random intercept
+#' modmat = make_matrices(~ s(g, bs = "re") + s(x, bs = "ps"), data)
+#' # tensorproduct of x and y
+#' modmat = make_matrices(~ s(x) + s(y) + ti(x,y), data)
+#' # multiple formulas at once
+#' modmat = make_matrices(list(mu ~ s(x) + y, sigma ~ s(g, bs = "re")), data = data)
+make_matrices <- function(formula, data, knots = NULL){
+  if(!is.list(formula)){
+    # not a list -> check if formula is a single formula
+    if(inherits(formula, "formula")){
+      # check knots
+      return(make_matrices_flat(formula, data, knots))
+    } else{
+      stop("'formula' must be a (nested) list of formulas or a single formula.")
+    }
+  } else if(all(sapply(formula, inherits, what = "formula"))){     # check if flat list
+    # check knots
+    if(!is.null(knots)){
+      if(!is.list(knots)){
+        stop("'knots' must be a list of knots for each formula in 'formula'.")
+      } else if(!all(sapply(knots, is.list))){
+        stop("'knots' must be a named list of knot lists, containing knots for the corresponding formula (top level) and variable (bottom level) in 'formula'.")
+      }
+    }
+    return(make_matrices_flat(formula, data, knots))
+  } else{
+    # check if two level list
+    nested_check <- all(sapply(formula, function(f){
+      all(sapply(f, inherits, what = "formula"))
+    }))
+    
+    if(nested_check){
+      Z_list <- list()
+      S_list <- list()
+      pardim_list <- list()
+      coef_list <- list()
+      gam_list <- list()
+      gam0_list <- list()
+      knots_list <- list()
+      
+      names <- names(formula)
+      if(is.null(names)){
+        names <- paste0("stream", seq_along(formula))
+      }
+      
+      for(i in seq_along(formula)){
+        thisname <- names[i]
+        res <- make_matrices_flat(formula[[i]], data, knots[[i]])
+        
+        # handle nameing of S and coef
+        if(length(res$S) > 0){
+          names(res$S) <- paste0(thisname, ".", names(res$S))
+        }
+        names(res$coef) <- paste0(thisname, ".", names(res$coef))
+        
+        Z_list[[thisname]] <- res$Z
+        S_list <- c(S_list, res$S)
+        pardim_list[[thisname]] <- res$pardim
+        coef_list <- c(coef_list, res$coef)
+        gam_list[[thisname]] <- res$gam
+        gam0_list[[thisname]] <- res$gam0
+        knots_list[[thisname]] <- res$knots
+      }
+      
+      out <- list(
+        Z = Z_list, 
+        S = S_list, 
+        pardim = pardim_list,
+        coef = coef_list,
+        data = data,
+        gam = gam_list, 
+        gam0 = gam0_list,
+        knots = knots_list
+      )
+      class(out) <- "LaMa_matrices"
+      return(out)
+    } else{
+      stop("'formula' must be a (nested) list of formulas or a single formula.")
+    }
+  }
+}
+
+# process_obs_formulas <- function(formulas, 
+#                                  dists,
+#                                  nStates){
+# 
+#   # get dists
+#   dist_fns <- lapply(dists, function(d){
+#     if(is.character("d")){
+#       get(paste0("d", d))
+#     } else if(is.function(d)){
+#       d
+#     }
+#   })
+#   # for each dist, find parameter names of dist
+#   parnames <- lapply(dist_fns, function(d) formals(d)[-1])
+#   
+#   
+#   # create nested named formula list with streami.par.i ~ 1 for all by default
+#   # match with provided formulas
+# }
+
+#' Process and standardise formulas for the state process of hidden Markov models
+#'
+#' @param formulas formulas for the transition process of a hidden Markov model, either as a single formula, a list of formulas, or a matrix.
+#' @param nStates number of states of the Markov chain
+#' @param ref optional vector of reference categories for each state, defaults to \code{1:nStates}. 
+#' If provided, must be of length \code{nStates} and contain valid state indices. 
+#' If a formula matrix is provided, this cannot be specified because reference categries are specified by one \code{"."} entry in each row.
+#'
+#' @returns named list of formulas of length \code{nStates * (nStates - 1)}, where each formula corresponds to a transition from state \eqn{i} to state \eqn{j}, excluding transitions from \code{i} to \code{ref[i]}.
+#' @export
+#'
+#' @examples
+#' # single formula for all non-reference category elements
+#' formulas = process_hid_formulas(~ s(x), nStates = 3)
+#' # now a list of length 6 with names tr.ij, not including reference categories
+#' 
+#' # different reference categories
+#' formulas = process_hid_formulas(~ s(x), nStates = 3, ref = c(1,1,1))
+#' 
+#' # different formulas for different entries (and only for 2 of 6)
+#' formulas = list(tr.12 ~ s(x), tr.23 ~ s(y))
+#' formulas = process_hid_formulas(formulas, nStates = 3, ref = c(1,1,1))
+#' # also a list of length 6, remaining entries filled with tr.ij ~ 1
+#' 
+#' # matrix input with reference categories
+#' formulas = matrix(c(".", "~ s(x)", "~ s(y)",
+#'                     "~ g", ".", "~ I(x^2)",
+#'                     "~ y", "~ 1", "."), 
+#'                     nrow = 3, byrow = TRUE)
+#' # dots define reference categories
+#' formulas = process_hid_formulas(formulas, nStates = 3)
+process_hid_formulas <- function(formulas, 
+                                 nStates,
+                                 ref = NULL){
+  
+  # extract formula names from response variable
+  get_name <- function(fml) {
+    if (length(fml) == 3 && !deparse(fml[[2]]) %in% c("", ".")) {
+      return(deparse(fml[[2]]))
+    } else {
+      return(NULL)
+    }
+  }
+  
+  # initialise transition names without reference categories
+  make_tr_names <- function(nStates, ref){
+    names <- c()
+    for(i in 1:nStates){
+      for(j in 1:nStates){
+        if(j != ref[i]){
+          names <- c(names, paste0("tr.", i, j))
+        }
+      }
+    }
+    names
+  }
+  
+  # NULL input → uniform ~1 formulas
+  if(is.null(formulas)){
+    formulas <- ~ 1
+  }
+  
+  # Matrix input: extract ref directly from "."
+  if(inherits(formulas, "matrix")){
+    if (!all(dim(formulas) == c(nStates, nStates))) {
+      stop("Formula matrix must be of size nStates x nStates.")
+    }
+    
+    ref <- integer(nStates)
+    for (i in 1:nStates) {
+      row <- formulas[i, ]
+      dot_count <- sum(trimws(row) == "." | trimws(row) == "~ .")
+      if (dot_count != 1) {
+        stop(sprintf("Row %d of formula matrix must contain exactly one '.' (reference transition), found %d.", i, dot_count))
+      }
+      ref[i] <- which(trimws(row) %in% c(".", "~ ."))
+    }
+  }
+  
+  # Default reference if not yet defined
+  if(is.null(ref)){
+    ref <- 1:nStates
+  } else{
+    if (length(ref) != nStates) stop("'ref' must be of length nStates.")
+    if (!all(ref %in% 1:nStates)) stop("'ref' must contain valid state indices.")
+  }
+  
+  # Generate transition names and empty formula list
+  names_out <- make_tr_names(nStates, ref)
+  formula_list <- stats::setNames(vector("list", length(names_out)), names_out)
+  
+  # Fill from matrix (now that ref is known)
+  if(inherits(formulas, "matrix")){
+    for (i in 1:nStates) {
+      for (j in 1:nStates) {
+        entry <- formulas[i, j]
+        if (trimws(entry) %in% c(".", "~ .")) next
+        if (j == ref[i]) next
+        
+        name <- paste0("tr.", i, j)
+        if (is.character(entry)) entry <- stats::as.formula(entry)
+        formula_list[[name]] <- entry
+      }
+    }
+    return(formula_list)
+  }
+  
+  # Uniform formula for all transitions
+  if(inherits(formulas, "formula")){
+    for(i in seq_along(formula_list)){
+      formula_list[[i]] <- formulas
+    }
+    return(formula_list)
+  }
+  
+  # Named or unnamed list
+  if(inherits(formulas, "list")){
+    form_names <- names(formulas)
+    if(is.null(form_names)){
+      form_names <- sapply(formulas, get_name)
+    }
+    if(is.list(form_names)){
+      stop("If formula list is provided, it must be a named list with names 'tr.ij'.")
+    }
+    if(!all(form_names %in% names_out)){
+      msg <- "A formula for a reference category was provided. Formulas should not be provided for:"
+      trs <- paste0("tr.", 1:nStates, ref, collapse = ", ")
+      msg <- paste(msg, trs)
+      stop(msg)
+    }
+    
+    for (n in names_out) formula_list[[n]] <- ~1
+    for (i in seq_along(formulas)) {
+      formula_list[[form_names[i]]] <- formulas[[i]]
+    }
+    return(formula_list)
+  }
+  
+  stop("Unsupported input type for 'formulas'. Must be a formula, list, or matrix.")
+}
+
+
+
+
+
 #' Build the prediction design matrix based on new data and model_matrices object created by \code{\link{make_matrices}}
 #'
 #' @param object model matrices object as returned from \code{\link{make_matrices}}
 #' @param newdata data frame containing the variables in the formula and new data for which to evaluate the basis
+#' @param what optional character string specifying which formula to use for prediction, if \code{object} contains multiple formulas. If \code{NULL}, the first formula is used.
 #' @param ... needs to be a \code{newdata} data frame containing the variables in the formula and new data for which to evaluate the basis
 #'
+#' @seealso \code{\link{make_matrices}} for creating objects of class \code{LaMa_matrices} which can be used for prediction by this function.
+#'
 #' @return prediction design matrix for \code{newdata} with the same basis as used for \code{model_matrices}
 #' @export
 #' 
 #'
 #' @examples
+#' # single formula
 #' modmat = make_matrices(~ s(x), data.frame(x = 1:10))
-#' Z_predict = predict(modmat, data.frame(x = 1:10 - 0.5))
-predict.LaMa_matrices <- function(object, newdata, ...){
+#' Z_p = predict(modmat, data.frame(x = 1:10 - 0.5))
+#' # with multiple formulas
+#' modmat = make_matrices(list(mu ~ s(x), sigma ~ s(x, bs = "ps")), data = data.frame(x = 1:10))
+#' Z_p = predict(modmat, data.frame(x = 1:10 - 0.5), what = "mu")
+#' # nested formula list
+#' form = list(stream1 = list(mu ~ s(x), sigma ~ s(x, bs = "ps")))
+#' modmat = make_matrices(form, data = data.frame(x = 1:10))
+#' Z_p = predict(modmat, data.frame(x = 1:10 - 0.5), what = c("stream1", "mu"))
+predict.LaMa_matrices <- function(object, newdata, what = NULL, ...){
   # dots <- list(...)
   # if(!is.null(dots$newdata)){
   #   newdata <- dots$newdata
@@ -250,7 +672,7 @@ predict.LaMa_matrices <- function(object, newdata, ...){
   #   newdata <- dots[[1]]
   # }
   
-  pred_matrix(object, newdata)
+  pred_matrix(object, newdata = newdata, what = what)
 }
 
 
@@ -258,6 +680,7 @@ predict.LaMa_matrices <- function(object, newdata, ...){
 #'
 #' @param model_matrices model_matrices object as returned from \code{\link{make_matrices}}
 #' @param newdata data frame containing the variables in the formula and new data for which to evaluate the basis
+#' @param what optional character string specifying which formula to use for prediction, if \code{object} contains multiple formulas. If \code{NULL}, the first formula is used.
 #' @param exclude optional vector of terms to set to zero in the predicted design matrix. Useful for predicting main effects only when e.g. \code{sd(..., bs = "re")} terms are present. See \code{mgcv::predict.gam} for more details.
 #' @return prediction design matrix for \code{newdata} with the same basis as used for \code{model_matrices}
 #' @export
@@ -267,14 +690,64 @@ predict.LaMa_matrices <- function(object, newdata, ...){
 #' @importFrom mgcv predict.gam
 #'
 #' @examples
+#' # single formula
 #' modmat = make_matrices(~ s(x), data.frame(x = 1:10))
-#' Z_predict = pred_matrix(modmat, data.frame(x = 1:10 - 0.5))
+#' Z_p = pred_matrix(modmat, data.frame(x = 1:10 - 0.5))
+#' # with multiple formulas
+#' modmat = make_matrices(list(mu ~ s(x), sigma ~ s(x, bs = "ps")), data = data.frame(x = 1:10))
+#' Z_p = pred_matrix(modmat, data.frame(x = 1:10 - 0.5), what = "mu")
+#' # nested formula list
+#' form = list(stream1 = list(mu ~ s(x), sigma ~ s(x, bs = "ps")))
+#' modmat = make_matrices(form, data = data.frame(x = 1:10))
+#' Z_p = pred_matrix(modmat, data.frame(x = 1:10 - 0.5), what = c("stream1", "mu"))
 pred_matrix = function(model_matrices, 
                        newdata,
+                       what = NULL,
                        exclude = NULL) {
-  gam_setup0 = mgcv::gam(model_matrices$formula, 
-                         data = cbind(dummy = 1, model_matrices$data),
-                         knots = model_matrices$knots)
+  
+  if(is.null(model_matrices$gam0)){
+    gam_setup0 = mgcv::gam(model_matrices$formula,
+                           data = cbind(dummy = 1, model_matrices$data),
+                           knots = model_matrices$knots)
+  } else {
+    if(inherits(model_matrices$gam0, "gam")){
+      if(!is.null(what)){
+        message("'model_matrics' only contains a single gam object, 'what' is not used.")
+      }
+      gam_setup0 <- model_matrices$gam0
+    } else {
+      if(!inherits(model_matrices$gam0[[1]], "gam")){
+        if(is.null(what)){
+          stop("'what' must be specified and contain a top-level name of the formula list and a name/ response variable from your original formulas.")
+        }
+        if(length(what) != 2 | !is.character(what)){
+          stop("'what' must be a character vector of length 2, containing the top-level name of the formula list and a name/ response variable from your original formulas.")
+        }
+        if(!what[1] %in% names(model_matrices$gam0)){
+          stop("'what[1]' must be a top-level name of your formula list.")
+        }
+        if(!what[2] %in% names(model_matrices$gam0[[what[1]]])){
+          stop("'what[2]' must be one of the names/ response variables of your original formulas.")
+        }
+        gam_setup0 <- model_matrices$gam0[[what[1]]][[what[2]]]
+      } else {
+        if(is.null(what)){
+          stop("'what' must be specified and be one of the names/ response variables from your original formulas.")
+          # what <- names(model_matrices$Z)[1]
+        }
+        if(!what %in% names(model_matrices$gam0)){
+          stop("'what' must be one of the names of your original formulas.")
+        }
+      }
+      
+      gam_setup0 <- model_matrices$gam0[[what]]
+    }
+  }
+  
+  # if pi in formula, mgcv requires pi column in data
+  if("pi" %in% all.names(gam_setup0$formula)){
+    newdata = cbind(newdata, pi)
+  }
   
   predict.gam(gam_setup0, 
               newdata = cbind(dummy = 1, newdata), 
@@ -567,25 +1040,30 @@ make_splinecoef = function(model_matrices,
   basis_pos = model_matrices$basis$basis_pos
   k = length(basis_pos)
   
+  # scaling
+  dens1 = rowSums(model_matrices$Z_predict)
+  scaling = sapply(1:k, function(i) dens1[which.min(abs(basis_pos[i] - model_matrices$xseq))])
+  
   if(type == "real"){ # if density has support on the reals -> use normal distribution to initialize
-    beta = sapply(basis_pos[-k], dnorm, mean = par$mean, sd = par$sd, log = TRUE)
+    beta = sapply(basis_pos, dnorm, mean = par$mean, sd = par$sd, log = TRUE)
+    if(is.vector(beta)) beta = matrix(beta, nrow = 1)
     # rescaling to account for non-equidistant knot spacing
-    beta = beta - log(apply(model_matrices$Z_predict[,-k], 2, max))
+    beta = t(t(beta) - log(apply(model_matrices$Z_predict, 2, max)))
   } else if(type == "positive") { # if density has support on positive continuous -> use gamma distribution
     # transformation to scale and shape
-    beta = sapply(basis_pos[-k], dgamma2, mean = par$mean, sd = par$sd, log = TRUE)
+    beta = sapply(basis_pos, dgamma2, mean = par$mean, sd = par$sd, log = TRUE)
+    if(is.vector(beta)) beta = matrix(beta, nrow = 1)
     # rescaling to account for non-equidistant knot spacing
-    beta = beta - log(apply(model_matrices$Z_predict[,-k], 2, max))
+    beta = t(t(beta) - log(apply(model_matrices$Z_predict, 2, max)))
   } else if(type == "circular") {
-    beta = sapply(basis_pos[-k], LaMa::dvm, mu = par$mean, kappa = par$concentration, log = TRUE)
+    beta = sapply(basis_pos, LaMa::dvm, mu = par$mean, kappa = par$concentration, log = TRUE)
+    if(is.vector(beta)) beta = matrix(beta, nrow = 1)
   }
-  
-  if(is.vector(beta)){
-    beta = matrix(beta, nrow = 1, ncol = length(beta))
-  }
-  beta = beta - beta[,k-1]
-  cat("Parameter matrix excludes the last column. Fix this column at zero!\n")
-  return(beta)
+
+  beta = beta - beta[,k]
+  # beta = beta - beta[,k-1]
+  cat("Parameter matrix excludes the last column. Add a (fixed) zero column using 'cbind(coef, 0)' in your loss function!\n")
+  return(beta[,-k])
 }
 
 #' Build the design and penalty matrices for smooth density estimation
