@@ -4,17 +4,22 @@
 #' including nonlinear functions of random effects and parameters.
 #' 
 #' @details
-#' \strong{Caution:} Currently does not work for models with fixed parameters (i.e. that use the \code{map} argument of \code{MakeADFun}.)
+#' This function simply samples from the approximate multivariate normal distribution of the maximum likelihood estimate (MLE) of the parameters
+#' \deqn{ \hat{\theta} \sim N(\theta, H^{-1}), }
+#' where \eqn{H} is the Hessian matrix of the negative log-likelihood function at the MLE. 
+#' It then returns either the sampled parameters or \code{REPORT()}ed transformations of them. If \code{CI = TRUE}, it does not return the samples but directly computes confidence intervals.
+#' 
+#' If you are interested in several quantities, calling \code{sdreportMC} once with a vector \code{what} will generally be faster than calling it several times with single elements of \code{what}.
 #'
-#' @param obj object returned by \code{MakeADFun()} after optimisation
-#' @param what vector of strings with names of parameters and \code{REPORT()}ed quantities to be reported
+#' @param obj object returned by \code{MakeADFun()} after optimisation or model of class \code{qremlModel} as returned by \code{\link{qreml}}.
+#' @param what vector of strings with names of parameters and/or \code{REPORT()}ed quantities to be reported
 #' @param nSamples number of samples to draw from the multivariate normal distribution of the MLE
 #' @param Hessian optional Hessian matrix. If not provided, it will be computed from the object
 #' @param CI logical. If \code{TRUE}, only confidence intervals instead of samples will be returned
 #' @param probs vector of probabilities for the confidence intervals (ignored if no CIs are computed)
 #'
 #' @return named list corresponding to the elements of \code{what}. Each element has the structure of the corresponding quantity with an additional dimension added for the samples.
-#' For example, if a quantity is a vector, the list contains a matrix. If a quantity is a matrix, the list contains an array.
+#' For example, if a quantity is a vector, the list contains a matrix. If a quantity is a matrix, the list contains an array. If quantity is an array, the list contains an array with one extra dimension.
 #' @export
 #' 
 #' @import RTMB
@@ -69,7 +74,7 @@
 #' # `mu` has report statement, `delta` is automatically reported by `forward()`
 #' sdrMC = sdreportMC(obj, 
 #'                    what = c("mu", "delta"), 
-#'                    n = 50)
+#'                    nSamples = 50)
 #' dim(sdrMC$delta)
 #' # now a matrix with 50 samples (rows)
 sdreportMC = function(obj, 
@@ -80,6 +85,14 @@ sdreportMC = function(obj,
                       probs = c(0.025, 0.975)){
   n <- nSamples
   
+  # check whether obj is either a qremlModel or an RTMB object
+  if(inherits(obj, "qremlModel")){
+    obj <- obj$obj # qremlModel has RTMB obj as entry named obj
+  }
+  if(is.null(obj$env)){
+    stop("'obj' seems not to be generated by 'RTMB::MakeADFun()'")
+  }
+  
   if(is.null(Hessian)){
     # check if Hessian can be computed
     # if not, obj is a marginal llk -> no Hessian implemented
@@ -99,14 +112,18 @@ sdreportMC = function(obj,
     sdr = sdreport(obj, ignore.parm.uncertainty = TRUE)
   }
   
+  # extract relisting function
+  relist_par = obj$env$parList
   # extract parameter list from sdreport
   parlist = as.list(sdr, "Estimate")
   # save parameter list skeleton to reshape later
-  skeleton = utils::as.relistable(parlist)
+  # skeleton = utils::as.relistable(parlist)
   # save parameter names
   parnames = names(parlist)
   # turn estimated parameter list into vector
-  parvec = unlist(parlist)
+  # parvec <- obj$env$par
+  parvec <- sdr$par.fixed
+  # parvec = unlist(parlist)
   # compute Fisher information
   I = ginv(as.matrix(H))
   
@@ -124,14 +141,16 @@ sdreportMC = function(obj,
   if(length(whatpar) > 0){
     if(length(whatpar) == 1){
       Samplepars1 = lapply(1:n, function(t){
-        samplelist = utils::relist(simpars[t,], skeleton)
+        # samplelist = utils::relist(simpars[t,], skeleton)
+        samplelist = relist_par(simpars[t,])
         samplelist[[whatpar]]})
       
       Samplepars = stats::setNames(vector("list", length(whatpar)), whatpar)
       Samplepars[[1]] = Samplepars1
     } else{
       Samplepars1 = lapply(1:n, function(t){
-        samplelist = utils::relist(simpars[t,], skeleton)
+        # samplelist = utils::relist(simpars[t,], skeleton)
+        samplelist = relist_par(simpars[t,])
         samplelist[whatpar]})
       
       # create list skeleton, first layer: what, second layer: samples
@@ -233,7 +252,10 @@ sdreportMC = function(obj,
         CIpar[[i]] = qthisSamplepars
       }
       names(CIpar) = names(Samplepars)
-    } else{CIpar = NULL}
+    } else{
+      CIpar = NULL
+    }
+    
     if(!is.null(Samplereports)){
       CIreport = list()
       for(i in 1:length(Samplereports)){
@@ -242,7 +264,7 @@ sdreportMC = function(obj,
           qthisSamplereports = stats::quantile(thisSamplereports, probs)
         } else if(is.matrix(thisSamplereports)){
           qthisSamplereports = apply(thisSamplereports, 2, stats::quantile, probs)
-        } else if(is.array(thisSamplepars) & !is.matrix(thisSamplepars)){
+        } else if(is.array(thisSamplereports) & !is.matrix(thisSamplereports)){
           qthisSamplereports = apply(thisSamplereports, 1:(length(dim(thisSamplereports))-1), stats::quantile, probs)
         }
         CIreport[[i]] = qthisSamplereports
